# -*- coding: utf-8 -*-
# this file is released under public domain and you can use without limitations

#########################################################################
## This scaffolding model makes your app work on Google App Engine too
#########################################################################

def connect_to_db():
    
    if request.env.web2py_runtime_gae:            # if running on Google App Engine
        db = DAL('gae')                           # connect to Google BigTable
        session.connect(request, response, db = db) # and store sessions and tickets there
        ### or use the following lines to store sessions in Memcache
        # from gluon.contrib.memdb import MEMDB
        # from google.appengine.api.memcache import Client
        # session.connect(request, response, db = MEMDB(Client()))
    else:                                         # else use a normal relational database
        db = DAL(DB_CONNECT_URI)       # if not, use SQLite or other DB

    return db

def init_mail():

    from gluon.tools import Mail

    mail = Mail()                                  # mailer
    mail.settings.server = MAIL_SERVER or 'smtp.gmail.com:587'  # your SMTP server
    mail.settings.sender = MAIL_SENDER         # your email
    mail.settings.login = MAIL_LOGIN      # your credentials or None

    return mail

def init_auth(globals, db):

    from gluon.tools import Auth    
    import datetime

    #8.1.4 Customizing Auth page 350
    auth = Auth(globals,db)      #authentication/authorization
    auth.settings.hmac_key = REGISTER_HMAC_KEY  # before define_tables()

    if REGISTER_METHOD in ['Disabled','Janrain']:
        auth.settings.actions_disabled.append('register') #disable register

    #Recaptcha (pag349)
    if REGISTER_METHOD in ['Recaptcha']:
        from gluon.tools import Recaptcha
        auth.settings.register_captcha = Recaptcha(request, RECAPTCHA_PUBLIC_KEY, RECAPTCHA_PRIVATE_KEY)    

    db.define_table('auth_user',
        Field('id','id',
              represent=lambda id:SPAN(id,' ',A('view',_href=URL('auth_user_read',args=id)))),
        Field('username', type='string',
              label=T('Username')),
        Field('first_name', type='string',
              label=T('First Name')),
        Field('last_name', type='string',
              label=T('Last Name')),
        Field('email', type='string',
              label=T('Email')),
        Field('password', type='password',
              readable=False,
              label=T('Password')),
        Field('manager', type='boolean', default=False,
              label=T('Manager'),writable=False,readable=False),
        Field('admin', type='boolean', default=False,
              label=T('Admin'),writable=False,readable=False),
        Field('created_on','datetime',default=request.now,
              label=T('Created On'),writable=False,readable=False),
        Field('modified_on','datetime',default=request.now,
              label=T('Modified On'),writable=False,readable=False,
              update=request.now),
        Field('registration_key',default='',
              writable=False,readable=False),
        Field('reset_password_key',default='',
              writable=False,readable=False),
        Field('registration_id',default='',
              writable=False,readable=False),
        Field('site_language', length=128,default='',
              label=T('Language'),writable=False,readable=False,),
        format='%(username)s',
        migrate=True)

    custom_auth_table = db[auth.settings.table_user_name] # get the custom_auth_table
    custom_auth_table.first_name.requires = IS_NOT_EMPTY(error_message=auth.messages.is_empty)
    custom_auth_table.last_name.requires =  IS_NOT_EMPTY(error_message=auth.messages.is_empty)
    custom_auth_table.password.requires = [CRYPT(key=auth.settings.hmac_key)]
    custom_auth_table.username.requires = IS_NOT_IN_DB(db, db.auth_user.username)
    custom_auth_table.email.requires = [
      IS_EMAIL(error_message=auth.messages.invalid_email),
      IS_NOT_IN_DB(db, custom_auth_table.email)]

    auth.settings.table_user = custom_auth_table

    auth.define_tables()         # creates all needed tables
    auth.settings.mailer = mail  # for user email verification

    if REGISTER_METHOD in ['None','Recaptcha','Approval']:
        auth.settings.registration_requires_verification = False
    else:
        auth.settings.registration_requires_verification = True

    if REGISTER_METHOD in ['Approval']:
        auth.settings.registration_requires_approval = True
    else:
        auth.settings.registration_requires_approval = False

    auth.settings.reset_password_requires_verification = True

    if REGISTER_METHOD in ['Janrain']:
        from gluon.contrib.login_methods.rpx_account import RPXAccount
        base_http = 'http://' + str(request.env.http_host)
        auth.settings.actions_disabled=['register','change_password','request_reset_password']
        auth.settings.login_form = RPXAccount(request, api_key=JANRAIN_API_KEY,domain=JANRAIN_DOMAIN,
         url = base_http + '/%s/default/user/login' % request.application)

    auth.messages.verify_email = 'Click on the link http://'+request.env.http_host+\
        URL(r=request,c='default',f='user',args=['verify_email'])+\
        '/%(key)s to verify your email'
    auth.messages.reset_password = 'Click on the link http://'+request.env.http_host+\
        URL(r=request,c='default',f='user',args=['reset_password'])+\
        '/%(key)s to reset your password'

    return auth
    
db = connect_to_db() #connect to database
mail = init_mail() #initialize mail
auth = init_auth(globals(), db) #initialize auth

from gluon.tools import *
crud = Crud(globals(),db)                      # for CRUD helpers using auth
service = Service(globals())                   # for json, xml, jsonrpc, xmlrpc, amfrpc
plugins = PluginManager()

crud.settings.auth = None                      # =auth to enforce authorization on crud

# creates a primes table for pagination
db.define_table('prime',Field('value','integer'))
def isprime(p):
    for i in range(2,p):
        if p%i==0: return False
    return True
if len(db().select(db.prime.id))==0:
   p=2
   for i in range(1000):
       while not isprime(p): p+=1
       db.prime.insert(value=p)
       p+=1