# -*- coding: utf-8 -*-
#
# Web2py Application Exhibition ( Version 2.0 ) 
# Copyright (c) 2010 Mulone, Pablo Martín 
# https://bitbucket.org/mulonemartin/exhibition
#
# License Code: GPL, General Public License v. 2.0
# License Content: Creative Commons Attribution 3.0 
#
# Also visit: www.web2py.com 
#             or Groups: http://groups.google.com/group/web2py 
#                http://groups.google.com/group/web2py-usuarios  
#
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>

import cStringIO
import os
import datetime
import time


#for language support
session.mylanguage = 'en';
if request.cookies.has_key('language'):
    session.mylanguage = request.cookies['language'].value
if session.mylanguage:
    T.force(session.mylanguage)


#for editing items and allowing embeds
def advanced_editor(field, value):
    return TEXTAREA(_id = str(field).replace('.','_'), _name=field.name, _class='text ckeditor', value=value, _cols=60, _rows=8)


db.list_item.description.widget = advanced_editor


def myXML(text): 
    return XML(text, sanitize=True, 
        permitted_tags=['a', 'b', 'blockquote', 'br/', 'i', 'li', 'ol', 'ul', 'p', 'cite', 'code', 'pre', 'img/','object','embed'], 
        allowed_attributes={'a':['href', 'title'], 'img':['src', 'alt'], 'blockquote':['type'],  'object':['height','width'], 'embed':['allowfullscreen','src','type'], 
    })


def timesince(d0, t0):
    chunks = (
      (60 * 60 * 24 * 365, lambda n: T('one year') if n==1 else T('%(n)d years',dict(n=n))),
      (60 * 60 * 24 * 30, lambda n: T('one month') if n==1 else T('%(n)d months',dict(n=n))),
      (60 * 60 * 24 * 7, lambda n: T('one week') if n==1 else T('%(n)d weeks',dict(n=n))),
      (60 * 60 * 24, lambda n: T('one day') if n==1 else T('%(n)d days',dict(n=n))),
      (60 * 60, lambda n: T('one hour') if n==1 else T('%(n)d hours',dict(n=n))),
      (60, lambda n: T('one minute') if n==1 else T('%(n)d minutes',dict(n=n))),      
      (1, lambda n: T('an instant') if n==1 else T('%(n)d seconds',dict(n=n))),
    )
    now = datetime.datetime.fromtimestamp(t0)
    d=d0
    delta = now - (d - datetime.timedelta(0, 0, d.microsecond))
    since = delta.days * 24 * 60 * 60 + delta.seconds
    if since <= 0: return T('an instant')
    for seconds, name in chunks:
        count = since // seconds
        if count != 0: break
    return name(count)
    

def url(f,args=[]): return URL(r=request,f=f,args=args)


def button(text,action,args=[]):
    return SPAN('[',A(text,_href=URL(r=request,f=action,args=args)),']')


def link_top_list(top_list):
    return A(top_list.name,_href=url('top_list',top_list.id))



#def top_list_items(top_list_id):
#    return db((db.top_list.id==top_list_id) & (db.list_item.id==db.lists.list_item))


def get_user_title(id):
           
    users = db(db.auth_user.id == id).select()    
    if users:
        user = users[0]
        user_title = user.last_name + ", " + user.first_name    
    
    return user_title


def likers_count(id):

    count = db(db.list_likes.top_list_id == id).count()   
    return count 


def voters_count(list_id,list_item_id):
    
    sub=db((db.top_list.id==db.list_votes.top_list_id) & (db.list_item.id==db.list_votes.list_item_id))
    count = sub(db.list_votes.list_item_id == list_item_id).count()
    return count


def get_list_likers(id):

    xmlusers=""    
    if is_user_an_admin():        
        users = ""
        likes = db(db.list_likes.top_list_id == id).select()    
        if likes:
            for like in likes:
                author = like.author_id
                user = '<li>%s</li>' % get_user_title(author)  
                users += user

            xmlusers = '<strong>Likers:</strong><ul>%s</ul>' % users              

    return xmlusers


def _has_privilege(top_list_id):   

    if is_user_an_admin():
        return True

    elif is_user_logged_in():    
        user_id = session.auth.user.id
        query = (db.top_list.id == top_list_id) & (db.top_list.user_id==user_id)
        ownlogo = db(query).select()
        if ownlogo:
            return True

    return False


def disqus_comments():   

    if DISQUS_DEVELOPER:
        developer = 'var disqus_developer = 1;';
    else:
        developer = '';

    script = '''
    <div id="disqus_thread"></div>
    <script type="text/javascript">
    %(developer)s
      /**
        * var disqus_identifier; [Optional but recommended: Define a unique identifier (e.g. post id or slug) for this thread] 
        */
      (function() {
       var dsq = document.createElement('script'); dsq.type = 'text/javascript'; dsq.async = true;
       dsq.src = 'http://%(site)s.disqus.com/embed.js';
       (document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0]).appendChild(dsq);
      })();
    </script>
    <noscript>Please enable JavaScript to view the <a href="http://disqus.com/?ref_noscript=%(site)s">comments powered by Disqus.</a></noscript>
    <a href="http://disqus.com" class="dsq-brlink">blog comments powered by <span class="logo-disqus">Disqus</span></a>

     ''' % {'developer': developer, 'site': DISQUS_SITE}
    return script


def disqus_comments_count():

    script = ''' 
    <script type="text/javascript">
    var disqus_shortname = '%(site)s';
    (function () {
      var s = document.createElement('script'); s.async = true;
      s.src = 'http://disqus.com/forums/%(site)s/count.js';
      (document.getElementsByTagName('HEAD')[0] || document.getElementsByTagName('BODY')[0]).appendChild(s);
    }());
    </script>
    ''' % {'site': DISQUS_SITE}

    return script


def disqus_last_comments():

    hide_avatars = 0        
    avatar_size = 48
    recent_comments=T("Recent Comments")
    num_items = 5

    script = '''
    <div id="recentcomments" class="dsq-widget">
    <h4 class="dsq-widget-title">%(recent_comments)s</h4>
    <script type="text/javascript" src="http://disqus.com/forums/%(site)s/recent_comments_widget.js?num_items=%(num_items)s&hide_avatars=%(hide_avatars)s&avatar_size=%(avatar_size)s&excerpt_length=200">
    </script>
    </div>
    <a href="http://disqus.com/">Powered by Disqus</a>
    '''  % {'site': DISQUS_SITE, 'recent_comments': recent_comments, 'avatar_size': avatar_size, 'hide_avatars': hide_avatars, 'num_items': num_items}

    return script


def pretty_exception(title="not found", message="this is the problem"):    

    img_warning = IMG(_src=URL(request.application,'static','images/warning.png'), _alt="Warning",_style="float: left; padding: 5px;")
    link_index = A(T('Back to the index page'), _href=URL(request.application,'default','index'))
    html =  '''
    <!DOCTYPE html>
    <html lang="en">
    <head>    
    <meta charset="utf-8">          
    <title>%(title)s</title>
    <style type="text/css">
    body {
        margin: 0;
        padding: 0;
        background: #fff;     
        font-family: Arial,Helvetica,sans-serif;     
        color: #444;
    }
    #wrapper {
        margin: 0 auto;
        padding: 0;
    }
    #page {
        width: 500px;
        margin: 0 auto;
        padding: 40px;        
    }
    #warning {       
        -webkit-border-radius: 5px;
        -moz-border-radius: 5px;
        border-radius: 5px;    
        width: 350px
        padding: 20px;        
        background-color: #fff;
        border: 1px solid #d0cfc2;            
        }
    .title {       
        font-size: 20px; 
        font-weight: bold;                   
        } 
    .content {        
        padding: 20px;           
        }
    .description {        
        font-size: 16px;             
        }     
    </style>
    </head>       
    <body>
    <div id="wrapper">
    <div id="page">
    <div id="warning"> 
        %(image)s 
        <div class="content"> 
        <div class="title"> %(title)s </div> 
        <div class="description">%(message)s <p>%(index)s</p> </div> 
        <div style="clear: both; float: none;"></div>
        </div>
    </div>
    </div>
    </div>
    </body>
    </html>    
    ''' % {'title': title, 'image': img_warning.xml(), 'message':message,'index':link_index.xml()}

    return html

if settings.is_first_time:
    first=db(db.auth_user.id>0).select()
    if first: #if there are no info in db, so i think is the first time and create default values
        settings.is_first_time=False
    else:
        my_crypt = CRYPT(key=auth.settings.hmac_key)
        crypt_pass = my_crypt(settings.default_admin_password)[0]        
        id_user= db.auth_user.insert(
                                   username=settings.default_admin_user,
                                   first_name=settings.default_admin_first_name,
                                   last_name=settings.default_admin_last_name,
                                   email = settings.default_admin_email,
                                   password = crypt_pass,
                                   manager = True,
                                   admin = True                            
                                   )